/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2005                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* 20041001  Initial Version                                         */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/* GENERAL INFORMATION:                                              */
/* Share Object name: qmac                                           */
/* Segment Name:      qmac23.cpp                                     */
/* Function Name:     movec_ret_func                                 */
/* Abstract:                                                         */
/* Description: Driver for tpf_movec_return(),tpf_movec_EVM_return() */
/*                                                                   */
/* Information: tpf_movec_return() moves data between an ECB virtual */
/*              memory(EVM) address space and a system virtual       */
/*              memory(SVM) address space.                           */
/*              tpf_movec_EVM_return() moves data from one ECB       */
/*              virtual memory address space to another EVM address  */
/*              space.                                               */
/*                                                                   */
/*              These two functions permit an ECB-controlled program */
/*              to read and modify storage that is not part of its   */
/*              own address space.                                   */
/*              Use this function to ensure that a system error does */
/*              not occur when moving the requested data.            */
/*  Format:                                                          */
/*    int  tpf_movec_return(conts void *fromaddr, int fromvm,        */
/*                          void * toaddr, int tovm, int length)     */
/*                                                                   */
/*    int  tpf_movec_EVM_return(const void *fromaddr, void *fromecb, */
/*                            void *toaddr, void *toecb, int length) */
/*                                                                   */
/*  Normal return:                                                   */
/*  	The number of bytes moved.                                     */
/*                                                                   */
/*  Error return:                                                    */
/*   	The number of bytes actually moved in an attempt to satisfy    */
/*    the request is returned and errno is set to one of the         */
/*    following:                                                     */
/*                                                                   */
/*    ETPFMOVE_BADFR:                                                */
/*       An address in the from range, specified by the fromaddr and */
/*       length parameters, is not valid. To determine the incorrect */
/*       address, add the return value to the value of the fromaddr  */
/*       parameter                                                   */
/*    ETPFMOVE_BADTO:                                                */
/*       An address in the to range, specified by the toaddr and     */
/*       length parameters, is not valid. To determine the incorrect */
/*       address, add the return value to the value of the toaddr    */
/*       parameter                                                   */
/*                                                                   */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>
#include <errno.h>
#include <qmac23.h>
#include <c_api5.h>

extern "C" void movec_ret_func() {

   struct ev0bk eb;
   eb.evnpstinf.evnbkc1 = 1;
   strcpy (eb.evnbkn, "cremcOK");
   struct parms_passed parameters;

   char *parent_eheap,
        *parent_eheap_passed,
        *from_mem,
        msg[MSG_SIZE],                     //MSG_SIZE = 120; TEXT_SIZE = 50
        text[TEXT_SIZE] = "abcdefghijklmnopqrstuvwxyz0123456789ABC",
        parent_stack_passed[SIZE_PASSED];  //SIZE_PASSED = 70000

   unsigned int rtval;

   void *sva,
        *sva_1,
        *parent_private_passed;

   int  fromint,
        rc,
        rc_1,
        out_of_map_sva = 300;

   // Gets two SVAs
   sva   = gsysc(1,"movec");
   sva_1 = gsysc(2,"movec_1");

   if (sva || sva_1)
      prtmsg("tpf_movec_return: Gets storage from the system heap successfully.\n");
   else  {
      prterr("tpf_movec_return: ERROR allocating system heap. Exits from here.");
      return;
   }

   parent_eheap_passed  = (char*) malloc(SIZE_PASSED);
   parent_eheap         = (char*) malloc(MALLOC_SIZE);  //MALLOC_SIZE=800
   from_mem             = (char*) malloc(MALLOC_SIZE);

   if ((! parent_eheap_passed) || (! from_mem) || (! parent_eheap))  {
      prterr("tpf_movec_return: malloc() error msg - Out of storage. Exits from here.");
      return;
   }

   parent_private_passed = getcc(DC, GETCC_TYPE, L2);   //attaches 1055-byte block

   // Initialize the parameters that used by the child ECB to test tpf_movec_EVM_return()
   parameters.is_errorCase        = ecbptr()->ebx000;
   parameters.printmsg            = ecbptr()->ebcm01;
   parameters.parent_addr_stack   = parent_stack_passed;      //test for stack
   parameters.parent_addr_eheap   = parent_eheap_passed;      //test for eheap
   parameters.parent_addr_private = parent_private_passed;    //test for private
   parameters.ECB_SVA_addr        = ecbp2()->ce2sva;
   strcpy(parameters.text, text);

   if (ecbptr()->ebx000 == FALSE)
   {
      /*********************************************/
      /* Testing NORMAL case of tpf_movec_return() */
      /*********************************************/
      prtmsg("Testing NORMAL cases of tpf_movec_return().\n");

      // Sets values
      memset(    from_mem, 'A', MALLOC_SIZE);
      memset(parent_eheap, '?', MALLOC_SIZE);

      // Moves data from EVA1 to SVA1
      prtmsg("tpf_movec_return: 1- Moves data from EVA1 to SVA1.\n");
      rtval = tpf_movec_return( from_mem,
                                TPF_MOVEC_EVA,
                                sva,
                                TPF_MOVEC_SVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg,  "tpf_movec_return: 1- %d bytes moved (intent length %d bytes).\n",
                 rtval,
                 MALLOC_SIZE);
         prterr(msg);
      }

      // Moves data from SVA1 to SVA2
      prtmsg("tpf_movec_return: 2- Moves data from SVA1 to SVA2.\n");
      rtval = tpf_movec_return( sva,
                                TPF_MOVEC_SVA,
                                sva_1,
                                TPF_MOVEC_SVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg,  "tpf_movec_return: 2- %d bytes moved (intent length %d bytes).\n",
                 rtval,
                 MALLOC_SIZE);
         prterr(msg);
      }

      // Moves data from SVA2 back to EVA2
      prtmsg("tpf_movec_return: 3- Moves data back from SVA2 to EVA2.\n");

      rtval = tpf_movec_return( sva_1,
                                TPF_MOVEC_SVA,
                                parent_eheap,
                                TPF_MOVEC_EVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg,  "tpf_movec_return: 3- %d bytes moved (intent length %d bytes).\n",
                 rtval,
                 MALLOC_SIZE);
         prterr(msg);
      }

      // Checks data moved.. Compares the contents of from_mem and parent_eheap
      if (memcmp(from_mem, parent_eheap, MALLOC_SIZE))  {
         prterr("tpf_movec_return: Error occurred in data moved. Exits from here.");
         return;
      }
      else
         prtmsg("tpf_movec_return: Data moved from EVM1->SVA1->SVA2->EVM2 successfully.\n");


      // out_of_map_sva to SVA
      prtmsg("tpf_movec_return: 4- Test From an out-of-map SVA address to mapped SVA.\n");
      rtval = tpf_movec_return( (void*)(long) out_of_map_sva,
                                 TPF_MOVEC_SVA,
                                 sva,
                                 TPF_MOVEC_SVA,
                                 FRAME_SIZE );

      if (rtval != FRAME_SIZE)   {
         sprintf(msg,  "tpf_movec_return: 4- %d bytes moved (intent length %d bytes).\n",
                 rtval,
                 FRAME_SIZE);
         prterr(msg);
      }

      // sva to EVM
      prtmsg("tpf_movec_return: 5- Test From SVA to EVM.\n");
      rtval = tpf_movec_return( sva,
                                TPF_MOVEC_SVA,
                                parent_eheap,
                                TPF_MOVEC_EVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)   {
         sprintf(msg,  "tpf_movec_return: 5- %d bytes moved (intent length %d bytes).\n",
                 rtval,
                 MALLOC_SIZE);
         prterr(msg);
      }

      if ( memcmp(sva, parent_eheap, MALLOC_SIZE) )  {
         prterr("tpf_movec_return: Error occurred in data moved. Exits from here.");
         return;
      }
      else
         prtmsg("tpf_movec_return: Data moved correctly.\n");

      // Move data from EVM to within the same EVM
      prtmsg("tpf_movec_return: 6- Moving data from EVM to within the same EVM.\n");
      memset(    from_mem, 'C', MALLOC_SIZE);
      memset(parent_eheap, '?', MALLOC_SIZE);

      rtval = tpf_movec_return( from_mem,
                                TPF_MOVEC_EVA,
                                parent_eheap,
                                TPF_MOVEC_EVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg,  "tpf_movec_return: 6- %d bytes moved (intent length %d bytes).\n",
                 rtval,
                 MALLOC_SIZE );
         prterr(msg);
      }

      if ( memcmp(from_mem, parent_eheap, MALLOC_SIZE) )  {
         prterr("tpf_movec_return: Error occurred in data moved. Exits from here.");
         return;
      }
      else
         prtmsg("Completes NORMAL cases of tpf_movec_return() successfully.\n");


      /**************************************************/
      /* Testing NORMAL cases of tpf_movec_EVM_return() */
      /**************************************************/

      // Resets the value
      memset(parent_eheap_passed, '?', MALLOC_SIZE);
      memset(parent_stack_passed, '?', MALLOC_SIZE);

      rc = evntc(&eb, EVENT_CNT, 'Y', 15, EVNTC_NORM);
      if (rc)  {
         prterr("10- Evntc() error: The event name already exists. Exits from here.");
         return;
      }

      // Create child ECB; Move data back to parent ECB's EVM
      prtmsg("CREMC a child ECB to test NORMAL cases of tpf_movec_EVM_return().\n");
      cremc(sizeof(struct parms_passed), &parameters, QMAI);

      prtmsg("Wait for the child ECB being created.\n");
      rc = evnwc(&eb, EVENT_CNT);
      if (rc) {
         prterr("10- Evnwc() error. Exits from here.");
         return;
      }

      // Checks data moved back
      prtmsg("tpf_movec_return: Checks the data moved back from child ECB.\n");
      if (memcmp(text, parent_stack_passed, sizeof(text)))  {
         prterr("tpf_movec_EVM_return: 7- Error occurred in data moved. Exits from here.");
         return;
      }

      if (memcmp(text, parent_eheap_passed, sizeof(text)))  {
         prterr("tpf_movec_EVM_return: 8- Error occurred in data moved. Exits from here.");
         return;
      }

      if (memcmp(text, parent_private_passed, sizeof(text)))  {
         prterr("tpf_movec_EVM_return: 9- Error occurred in data moved. Exits from here.");
         return;
      }

      prtmsg("tpf_movec_return: Data moved correctly.\n");
      prtmsg("Completes NORMAL cases of tpf_movec_EVM_return() successfully.\n");

   }
   else
   {

      /********************************************/
      /* Testing ERROR case of tpf_movec_return() */
      /********************************************/
      prtmsg("Testing ERROR cases of tpf_movec_return().\n");

      // test case 11: Bad From-SVA (error lst input)
      prtmsg("tpf_movec_return: 11- Testing error From-SVA address.\n");
      rtval = tpf_movec_return( (void *) INVALID_SVM,
                                TPF_MOVEC_SVA,
                                parent_eheap,
                                TPF_MOVEC_EVA,
                                MALLOC_SIZE);

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg, "tpf_movec_return: 11- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
         if (errno != ETPFMOVE_BADFR)
            prterr("tpf_movec_return: 11- Error errno. Should be ETPFMOVE_BADFR(5050).");
      }
      else
         prterr("tpf_movec_return: 11- Function did not fail with error input.");

      // test case 12: Bad From-SVA range
      prtmsg("tpf_movec_return: 12- Testing error From-SVA Range.\n");
      rtval = tpf_movec_return( (void *)(INVALID_SVM - MALLOC_SIZE),
                                TPF_MOVEC_SVA,
                                parent_eheap,
                                TPF_MOVEC_EVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg, "tpf_movec_return: 12- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
         if (errno != ETPFMOVE_BADFR)
            prterr("tpf_movec_return: 12- Error errno. Should be ETPFMOVE_BADFR(5050).");
      }
      else
         prterr("tpf_movec_return: 12- Function did not fail with error input.");

      // test case 13: Bad To-SVA (error 3rd input)
      prtmsg("tpf_movec_return: 13- Testing error To-SVA address.\n");
      rtval = tpf_movec_return( from_mem,
                                TPF_MOVEC_EVA,
                                (void *) INVALID_SVM,
                                TPF_MOVEC_SVA,
                                TEXT_SIZE );

      if (rtval != TEXT_SIZE)  {
         sprintf(msg, "tpf_movec_return: 13- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
         if (errno != ETPFMOVE_BADTO)
            prterr("tpf_movec_return: 13- Error errno. Should be ETPFMOVE_BADTO (5051).");
      }
      else
         prterr("tpf_movec_return: 13- Function did not fail with error input.");


      // test case 14: Bad To-SVA Range
      prtmsg("tpf_movec_return: 14- Testing error To-SVA Range.\n");
      rtval = tpf_movec_return( from_mem,
                                TPF_MOVEC_EVA,
                                (void *)(INVALID_SVM - MALLOC_SIZE),
                                TPF_MOVEC_SVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg, "tpf_movec_return: 14- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
         if (errno != ETPFMOVE_BADTO)
            prterr("tpf_movec_return: 14- Error errno. Should be ETPFMOVE_BADTO (5051)");
      }
      else
         prterr("tpf_movec_return: 14- Function did not fail with error input.");


      // test case 15: Incorrect From-EVA (error 1st input)
      prtmsg("tpf_movec_return: 15- Testing error From-EVM address.\n");
      rtval = tpf_movec_return( (void*) INVALID_EVM,
                                TPF_MOVEC_EVA,
                                sva,
                                TPF_MOVEC_SVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg, "tpf_movec_return: 15- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
         if (errno != ETPFMOVE_BADFR)
            prterr("tpf_movec_return: 15- Error errno. Should be ETPFMOVE_BADFR (5050).");
      }
      else
         prterr("tpf_movec_return: 15- Function did not fail with error input.");

      // test case 15a: Incorrect From-EVM (error 1st input)
      prtmsg("tpf_movec_return: 15a- Testing error From-EVM address.\n");
      rtval = tpf_movec_return( (void*) (INVALID_EVM - MALLOC_SIZE),
                                TPF_MOVEC_EVA,
                                sva,
                                TPF_MOVEC_SVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg, "tpf_movec_return: 15a- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
         if (errno != ETPFMOVE_BADFR)
            prterr("tpf_movec_return: 15a- Error errno. Should be ETPFMOVE_BADFR (5050).");
      }
      else
         prterr("tpf_movec_return: 15a- Function did not fail with error input.");


      // test case 16: Incorrect To-EVA (error 3rd input)
      prtmsg("tpf_movec_return: 16- Testing error To-EVM address.\n");
      rtval = tpf_movec_return( sva,
                                TPF_MOVEC_SVA,
                                (void *) INVALID_EVM,
                                TPF_MOVEC_EVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg, "tpf_movec_return: 16- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
         if (errno != ETPFMOVE_BADTO)
            prterr("tpf_movec_return: 16- Error errno. Should be ETPFMOVE_BADTO (5051)");
      }
      else
         prterr("tpf_movec_return: 16- Function did not fail with error input.");

      // test case 16a: Incorrect To EVA Range (error 3rd input)
      prtmsg("tpf_movec_return: 16a- Testing error To-EVM address.\n");
      rtval = tpf_movec_return( sva,
                                TPF_MOVEC_SVA,
                                (void*) (INVALID_EVM - MALLOC_SIZE),
                                TPF_MOVEC_EVA,
                                MALLOC_SIZE );

      if (rtval != MALLOC_SIZE)  {
         sprintf(msg, "tpf_movec_return: 16a- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
         if (errno != ETPFMOVE_BADTO)
            prterr("tpf_movec_return: 16a- Error errno. Should be ETPFMOVE_BADTO (5051)");
      }
      else
         prterr("tpf_movec_return: 16a- Function did not fail with error input.");

      /**************************************************/
      /* Testing ERROR cases of tpf_movec_EVM_return()  */
      /**************************************************/

      memset(parent_eheap, 'B', MALLOC_SIZE);

      rc = evntc(&eb, EVENT_CNT, 'Y', 15, EVNTC_NORM);
      if (rc) {
         prterr("31- Evntc() error: The event name already exists.");
         return;
      }

      // Tests error cases of different EVM address space
      prtmsg("CREMC an ECB to test ERROR cases of tpf_movec_EVM_return().\n");
      cremc(sizeof(struct parms_passed), &parameters, QMAI);

      prtmsg("Wait for the child ECB being created.\n");
      rc = evnwc(&eb, EVENT_CNT);
      if (rc) {
         prterr("32- Evnwc() error. Exits from here.");
         return;
      }

      // Out of bound range in the same EVM address space
      rtval = tpf_movec_EVM_return( &fromint,
                                    (void*) ecbp2()->ce2sva,
                                    &fromint,
                                    (void*) ecbp2()->ce2sva,
                                    150000 );

      if (rtval != 150000)  {
         sprintf(msg, "tpf_movec_EVM_return: 36- rc: %d, %s\n"
                    , rtval
                    , strerror(errno));
         prtmsg(msg);
      }
      else
         prterr("tpf_movec_return: 36- Function did not fail with error input.");

      prtmsg("Completed ERROR test cases successfully.\n");
   }

   // Releases system heap
   rc = rsysc(sva, 1, "movec");
   rc_1 = rsysc(sva_1, 2, "movec_1");
   if (rc != RSYSC_OK || rc_1 != RSYSC_OK)
      prterr("tpf_movec_return: ERROR occurred in releasing system heap.");

   // cleanup
   free(parent_eheap_passed);
   free(parent_eheap);
   free(from_mem);
   relcc(DC);

   prtmsg("Tests completed; Driver exited.\n");
   return;
}
